// mut_excl.h
// 
// Purpose: Guarantee that while a device is being modified, it
//          is not possible for another task to try to modify it.

// Description: This service prevents another task from accessing
//          a device by disabling interrupts and thus preventing
//          another task from accessing the device.  After the
//          device has been accessed, the interrupt flag is restored
//          to its previous state.  (ie., interrupts may have been
//          disabled at the beginning of this process, in which case
//          they will disabled when this routine completes).
//          Mutual_Exclusion may not be invoked multiple times
//          simultaneously.  (This is possible only if the same task
//          attempts it since this guarantees exclusive use of the
//          hardware by disabling interrupts to the CPU and not other
//          task could be active to invoke it).
// *********************************************************************

#if !defined(__MUT_EXCL)
#define __MUT_EXCL

#include <iostream.h>
#include "leg.h"

class Mutual_Exclusion
{

   unsigned int Previous_State;     // 16 bits for flags reg.
   unsigned char Service_Is_In_Use;

public:

   Mutual_Exclusion () {Service_Is_In_Use = FALSE;};

   void Save_Int_State_And_Disable_Int ();
      // This saves current interrupt flag state and prevents other
      // tasks from being activated.

   void Restore_Previous_Int_Flag ();
      // This restores the interrupt flag to its previous state
      // (possibly allowing other tasks to run).

};

#endif
